/*
 * Copyright 1999-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres and Dale Southard
 * 
 * This file is part of the Notre Dame Password Storage and Retrieval
 * (PSR) system.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame PSR system along with the software; see the file LICENSE.  If
 * not, contact Office of Research, University of Notre Dame, Notre Dame,
 * IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 * $Id: afskerbauth.c,v 1.7 2000/12/20 15:59:30 jsquyres Exp $
 *
 * Provides routines necessary for authenticating into AFS using the
 * kerb libraries available from ftp://ftp.pdc.kth.se/pub/krb/src and
 * elsewhere.
 *
 * Authenticate (const char *ln, const char *pw) Does the actual
 * authentication.  It takes pointers to strings with the desired AFS
 * login name and AFS password.  Returns 0 on success.
 *
 * AuthLife (void) Calculates the life of the token to the hour
 * (rounds down) and returns it as an int.  Returns -1 on failure.
 * 
 * GetNewPAG() gets a new PAG free of tokens.
 * 
 */

#include "psr_config.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#if HAVE_SYS_IOCCOM_H
#include <sys/ioccom.h>
#endif

#include <sys/ioctl.h>
#include <fcntl.h>
#include <limits.h>
#include <krb.h>
#include <kafs.h>
#include <stdlib.h>

#include "psr.h"


/* The Authenticate routine.  For AFS we first get a kerb tkt and then
   use the krb_afslog to get an AFS token Return 0 on success. */

int 
Authenticate(char *ln, char *pw)
{
  char tfile[PATH_MAX];
  char env[PATH_MAX + 11];
  int tf;
  int len;
  krb_principal princ;

  if (krb_parse_name(ln, &princ))
    return (1);
/* JMS Did this to make it work on Solaris 7 / 64 bit -- split
   addition of TKT_ROOT and %u_%u into two snprintfs */
  snprintf(tfile, sizeof(tfile), "%s", TKT_ROOT);
  len = strlen(tfile);
  snprintf(tfile + len, sizeof(tfile) - len, "%u_%u",
	   (unsigned) getuid(),
	   (unsigned) (getpid()));
  unlink(tfile);
  tf = open(tfile, O_CREAT | O_EXCL | O_RDWR);
  if (tf < 0) {
    return (1);
  }
  close(tf);
  unlink(tfile);
  snprintf(env, sizeof(env), "KRBTKFILE=%s", tfile);
  putenv(env);
  krb_set_tkt_string(tfile);
  if (princ.realm[0] == '\0')
    if (krb_get_lrealm(princ.realm, 1) != KSUCCESS)
      strcpy(princ.realm, KRB_REALM);
  if (krb_get_pw_in_tkt(princ.name, princ.instance, princ.realm,
	       KRB_TICKET_GRANTING_TICKET, princ.realm, DEFAULT_TKT_LIFE,
			pw)) {
    printf("Authenticate failed\n");
    return (1);
  }
  if (k_hasafs())
    krb_afslog(NULL, NULL);
  return (0);
}

/* The AuthLife routine. Mostly lifted from klist. */

int 
AuthLife(void)
{
  int life_hours;
  unsigned char t[128];
  struct ViceIoctl parms;
  u_int32_t i = 0;
  int32_t size_secret_tok, size_public_tok;
  struct ClearToken ct;
  unsigned char *r = t;

  parms.in = (char *) &i;
  parms.in_size = sizeof(i);
  parms.out = (char *) t;
  parms.out_size = sizeof(t);

  if (k_pioctl(NULL, VIOCGETTOK, &parms, 0) != 0) {
    return (-1);
  }

  /* figure out how big sec token is and skip past it */
  memcpy(&size_secret_tok, r, sizeof(size_secret_tok));
  r += size_secret_tok + sizeof(size_secret_tok);

  /* copy the public token into ct */
  memcpy(&size_public_tok, r, sizeof(size_public_tok));
  r += sizeof(size_public_tok);
  memcpy(&ct, r, size_public_tok);

  life_hours = (ct.EndTimestamp - time(NULL)) / 3600;
  return (life_hours);
}


/* GetNewPAG() -- gets a new PAG */

void 
GetNewPAG()
{
  if (k_hasafs())
    k_setpag();
}
