/*
 * Copyright 1999-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres and Dale Southard
 * 
 * This file is part of the Notre Dame Password Storage and Retrieval
 * (PSR) system.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame PSR system along with the software; see the file LICENSE.  If
 * not, contact Office of Research, University of Notre Dame, Notre Dame,
 * IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 * $Id: crypto.c,v 1.5 2000/12/20 15:59:06 jsquyres Exp $
 *
 * The pbs_authr crypto routines.  This version makes use of the
 * RSAREF v2 library.  
 *
 */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <global.h>
#include <rsaref.h>

#include "psr.h"
#include "md5.h"

#define MD5_HASH_LENGTH   16

/* DecryptPassword() -- Called with an uninit unsigned char pointer where the
password will be stored, plus const ponters to the dauthr_epass_header
and the private dauthr_key structures.  Returns 0 on success.  */

int 
DecryptPassword(char **clearpass, dauthr_epass * epass,
		const dauthr_key * privkey)
{
  R_ENVELOPE_CTX context;
  unsigned int partoutlength1, partoutlength2;

  *clearpass = (char*) malloc(epass->encrypted_pass_length + 7);
  if (*clearpass == NULL)
    return (1);
  if (R_OpenInit(&context,
		 ALGORITHM,
		 epass->encrypted_key,
		 epass->encrypted_key_length,
		 epass->iv,
		 (R_RSA_PRIVATE_KEY *) (&privkey->rsakey)))
    return (1);
  if (R_OpenUpdate(&context,
		   (unsigned char *) *clearpass,
		   &partoutlength1,
		   epass->encrypted_pass,
		   epass->encrypted_pass_length))
    return (1);
  if (R_OpenFinal(&context,
		  (unsigned char *) *clearpass + partoutlength1,
		  &partoutlength2))
    return (1);
  *(*clearpass + partoutlength1 + partoutlength2) = NULL;
  return (0);
}


/* EncryptPassword() -- Called with a pointer to a dauthr_epass struct
and const pointers to the cleartext password string, the R_RANDOM_STRUCT,
and the public dauthr_key.  Returns 0 on success */


int 
EncryptPassword(dauthr_epass * epass, const char *clearpass,
	   const dauthr_key * pubkey, R_RANDOM_STRUCT * random_structure)
{
  R_ENVELOPE_CTX context;
  unsigned char *encrypted_key;
  unsigned int passlength, partoutlength;
  R_RSA_PUBLIC_KEY *rsapubkey;

  passlength = ((unsigned int) strlen(clearpass));
  rsapubkey = (R_RSA_PUBLIC_KEY *) & pubkey->rsakey;
  encrypted_key = epass->encrypted_key;
  if (R_SealInit(&context,
		 &encrypted_key,
		 &epass->encrypted_key_length,
		 epass->iv,
		 1,
		 &rsapubkey,
		 ALGORITHM,
		 random_structure))
    return (1);
  if (R_SealUpdate(&context,
		   epass->encrypted_pass,
		   &partoutlength,
		   (unsigned char *) clearpass,
		   passlength))
    return (1);
  epass->encrypted_pass_length = partoutlength;
  if (R_SealFinal(&context,
		  epass->encrypted_pass + partoutlength,
		  &partoutlength))
    return (1);
  epass->encrypted_pass_length += partoutlength;
  return (0);
}


/* GenerateKeys() -- called with pointers to dauthr_pubkey
and dauthr_privkey, a const char pointer to the keyname, and a const
R_RANDOM_STRUCT ponter to a random structure.  Populates dauthr_pubkey
and dauthr_privkey and returns 0 on success */

int 
GenerateKeys(dauthr_key * pubkey, dauthr_key * seckey,
	     const char *keyname, R_RANDOM_STRUCT * random_structure)
{
  R_RSA_PROTO_KEY proto_key;

  proto_key.bits = KEYLENGTH;
  proto_key.useFermat4 = USERFERMAT4;
  if (R_GeneratePEMKeys((R_RSA_PUBLIC_KEY *) & pubkey->rsakey,
			(R_RSA_PRIVATE_KEY *) & seckey->rsakey,
			&proto_key,
			random_structure))
    return (1);
  if (strncpy(pubkey->keyname, keyname, MAXKEYNAME) == NULL)
    return (1);
  if (strncpy(seckey->keyname, keyname, MAXKEYNAME) == NULL)
    return (1);
  pubkey->keyname[MAXKEYNAME - 1] = seckey->keyname[MAXKEYNAME - 1] = NULL;
  return (0);
}


/* InitRandomStructure() -- Called with  pointer to a R_RANDOM_STRUCT,
populates the random structure and returns 0 on success.  Uses repeated
md5 hashes of ENTROPYSRC to get seed bytes.
 */

int 
InitRandomStructure(R_RANDOM_STRUCT * random_structure)
{
  unsigned int bytes_needed, remain, buflen;
  unsigned char seed_byte[MD5_HASH_LENGTH];
  unsigned char buf[BUFSIZ];
  FILE *entropy;
  MD5_CTX md;

  if (R_RandomInit(random_structure))
    return (1);
  remain = MD5_HASH_LENGTH;
  while (1) {
    if (remain == MD5_HASH_LENGTH &&
	(entropy = popen(ENTROPYSRC, "r")) != NULL) {
      MD5Init(&md);
      while ((buflen = fread(buf, 1, BUFSIZ, entropy)) > 0)
	MD5Update(&md, buf, buflen);
      MD5Final(seed_byte, &md);
      (void) pclose(entropy);
      remain = 0;
    }
    if (R_GetRandomBytesNeeded(&bytes_needed, random_structure))
      return (1);
    if (bytes_needed == 0)
      break;
    if (R_RandomUpdate(random_structure, &seed_byte[remain++], 1))
      return (1);
  }
  return (0);
}

/* DestroyRandomStructure() -- Called with a pointer to a R_RANDOM_STRUCT,
zeroizes the structer to prevent tampering. */

void 
DestroyRandomStructure(R_RANDOM_STRUCT * random_structure)
{
  R_RandomFinal(random_structure);
}
