/*
 * Copyright 1999-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres and Dale Southard
 * 
 * This file is part of the Notre Dame Password Storage and Retrieval
 * (PSR) system.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame PSR system along with the software; see the file LICENSE.  If
 * not, contact Office of Research, University of Notre Dame, Notre Dame,
 * IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 * $Id: dauth_shepherd.c,v 1.7 2000/12/20 15:58:48 jsquyres Exp $
 *
 * dauth_shepherd.c -- decrypts user password and uses it to
 * authenticate into AFS, then sleeps and repeats as necessary to
 * renew the * AFS tokens.
 * 
 */

#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>

#include <global.h>
#include <rsaref.h>
#include "psr.h"


int
main(int argc, char *argv[])
{
  int i;
  int verbose = 0;
  dauthr_key secret_key;
  dauthr_epass epass;
  char *pw_name = NULL, *clearpass, epass_file[PATH_MAX];
  int token_life_hours;
  pid_t pid;

  /* We're coming from another process, so close all file
     descriptors. */

  for (i = 3; i < sizeof(fd_set) * 8; i++)
    close(i);

  /* First we should process any commmand line arguements */

  for (i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-v") == 0)
      verbose = 1;
    else if (strcmp(argv[i], "-h") == 0) {
      (void) fprintf(stderr, "usage: dauth_shepherd [-v] username\n");
      exit(1);
    } else {
      if (pw_name == NULL)
	pw_name = strdup(argv[i]);
      else {
	(void) fprintf(stderr, 
		       "Warning: multiple usernames specified.  Exiting.\n");
	exit(1);
      }
    }
  }

  /* Load the secret key into memory. */

  if (LoadKeyFile(&secret_key, SECRETKEYFILE)) {
    if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd couldn't load secret key file \"%s\"\n",
		     SECRETKEYFILE);
    exit(1);
  }

  /* Find the Encrypted password file and if it exists load it into memory */

  if (DefaultEPassFile(epass_file, pw_name, secret_key.keyname)) {
    if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd could not find encrypted password file for key \"%s\"\n", 
		     secret_key.keyname);
    exit(1);
  }
  if (LoadEPassFile(&epass, epass_file)) {
    if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd could not load encrypted password file for key \"%s\"\n",
		     secret_key.keyname);
    exit(1);
  }

  /* Now call the crypto routine to decrypt the password */

  if (DecryptPassword(&clearpass, &epass, &secret_key)) {
    if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd could not decrypt encrypted password for key \"%s\"\n",
		     secret_key.keyname);
    exit(1);
  }

  /* Do Authentication. */

  if (Authenticate(pw_name, clearpass) != 0) {
    memset(clearpass, 0, MAXPASSLENGTH);
    if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd could not authenticate with password (wrong password?).\n");
    exit(1);
  }
  memset(clearpass, 0, MAXPASSLENGTH);
  free(clearpass);

  /* Get our token lifetime in hours */

  token_life_hours = AuthLife() - 1;
  if (token_life_hours < 1) {
    (void) fprintf(stderr, "dauth_shepherd found a token life of less than an hour\n");
    exit(1);
  }

  /* fork to become daemon, return our PID to the calling proc */

  pid = fork();
  if (pid < 0) {
    exit(1);
  }
  if (pid > 0) {
    printf("%i\n", pid);
    exit(0);
  }

  /* child reauthenticates forever */

  while (1) {
    (void) sleep((unsigned) abs(token_life_hours * 3600));
    if (LoadEPassFile(&epass, epass_file) != 0) {
      if (verbose)
	(void) fprintf(stderr, 
		       "dauth_shepherd could not load encrypted password file\n");
      exit(1);
    }
    if (DecryptPassword(&clearpass, &epass, &secret_key) != 0) {
      if (verbose)
	(void) fprintf(stderr, 
		       "dauth_shepherd could not decrypt encrypted password for key \"%s\"\n", 
		       secret_key.keyname);
      exit(1);
    }
    if (Authenticate(pw_name, clearpass) != 0) {
      memset(clearpass, 0, epass.encrypted_pass_length + 7);
      if (verbose)
      (void) fprintf(stderr, 
		     "dauth_shepherd could not authenticate with passowrd (wrong password?)\n");
      exit(1);
    }

    memset(clearpass, 0, epass.encrypted_pass_length + 7);
    free(clearpass);
  }

  /* Never gets here -- this program will be killed by an outside
     source */
}
