/*
 * Copyright 1999-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres and Dale Southard
 * 
 * This file is part of the Notre Dame Password Storage and Retrieval
 * (PSR) system.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame PSR system along with the software; see the file LICENSE.  If
 * not, contact Office of Research, University of Notre Dame, Notre Dame,
 * IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 * $Id: mkpsrauthrkeys.c,v 1.5 2000/12/20 15:57:18 jsquyres Exp $
 *
 * main src for the mkpbsauthrkeys program.
 * 
 * mkpbsauthrkeys is used to generate a pulic-secret keypair.  It
 * requires a ``name'' for the keys (this is usually the hostname).
 * Both keys are tagged with the name to provide a way for setting up
 * automatic key selection by scripts, etc.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <global.h>
#include <rsaref.h>

#include "psr.h"


int
main(int argc, char *argv[])
{
  dauthr_key public_key, secret_key;
  char *public_key_file = NULL;
  char *secret_key_file = SECRETKEYFILE;
  char *keyname = NULL;
  int i;
  int errflg = 0;
  int verbose = 0;
  R_RANDOM_STRUCT random_structure;

  /* First we should process any commmand line arguements */
  while ((i = getopt(argc, argv, "n:p:s:v")) != -1)
    switch (i) {
    case 'n':
      keyname = strdup(optarg);
      break;
    case 'p':
      public_key_file = strdup(optarg);
      break;
    case 's':
      secret_key_file = strdup(optarg);
      break;
    case 'v':
      verbose++;
      break;
    case '?':
      errflg++;
    }
  if (optind != argc) {
    errflg++;
  }
  if (errflg) {
    (void) fprintf(stderr,
		   "usage: mkpsrauthrkeys [-v] [-p publickeyfile ]"
		   " [-s secretkeyfile] [-n keyname]\n");
    exit(2);
  }
  /* Set the name of the key to the current hostname if not set. */
  if ((keyname == NULL) || (strlen(keyname) > MAXKEYNAME)) {
    keyname = (char*) malloc(MAXKEYNAME);
    if (keyname == NULL)
      exit(1);
    if (gethostname(keyname, MAXKEYNAME - 1)) {
      (void) fprintf(stderr,
		     "Couldn't get hostname, exiting.\n");
      exit(1);
    }
  }
  /* Print the keyname and secret keyfilename if we are verbose */
  if (verbose)
    (void) printf("Keyname is %s.\n", keyname);
  if (verbose)
    (void) printf("Secret keyfile is %s.\n", secret_key_file);


  /* Set the Public Key File (if not assigned) */

  if (public_key_file == NULL) {
    public_key_file = (char*) malloc(strlen(keyname) + strlen(PUBKEYSUFFIX)
				     + 1);
    if (strcpy(public_key_file, keyname) == NULL)
      return (1);
    if (strcat(public_key_file, PUBKEYSUFFIX) == NULL)
      return (1);
  }
  if (verbose)
    (void) printf("Public keyfile is %s.\n", public_key_file);


  /* Initialize the Random Structure */
  if (InitRandomStructure(&random_structure)) {
    (void) fprintf(stderr, "Couldn't initialize random"
		   " structure.\n");
    exit(1);
  }
  if (verbose)
    (void) printf("Random structure initialized.\n");

  /* Now call the crypto routine to generate the keys. */

  if (verbose)
    (void) printf("Generating key (this may take a little while)...\n");
  if (GenerateKeys(&public_key, &secret_key, keyname, &random_structure)) {
    (void) fprintf(stderr, "Key generation failed.\n");
    exit(1);
  }
  if (verbose)
    (void) printf("Key generation complete.\n");


  /* Save the public key file */

  if (SaveKeyFile(&public_key, public_key_file)) {
    (void) fprintf(stderr, "Failed to safe public key, "
		   "exiting.\n");
    exit(1);
  }
  /* Make sure the secret key file is secret then save it.  We can ignore
   * the return value of the chmod call because we don't care --- any
   * errors will cause a bomb during write, and if if the file hasn't been
   * created, the umask will handle it. */

  (void) umask(S_IRWXG + S_IRWXO);
  (void) chmod(public_key_file, S_IRUSR + S_IWUSR);
  if (SaveKeyFile(&secret_key, secret_key_file)) {
    (void) fprintf(stderr, "Failed to save secret key, "
		   "exiting.\n");
    exit(1);
  }
  if (verbose)
    (void) printf("Keys saved.\n");


  /* Destroy the random structure */

  DestroyRandomStructure(&random_structure);

  return 0;
}
