/* 
 * Copyright 1999-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres and Dale Southard
 * 
 * This file is part of the Notre Dame Password Storage and Retrieval
 * (PSR) system.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame PSR system along with the software; see the file LICENSE.  If
 * not, contact Office of Research, University of Notre Dame, Notre Dame,
 * IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 * $Id: pwstore.c,v 1.5 2000/11/14 15:41:00 jsquyres Exp $
 *
 * main src for the pwstore program.
 *
 * pwstore is used to encrypt the user's password with the specified
 * public key and store it.  It is the ``client side'' of the dauthr
 * package.
 *
 */

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <global.h>
#include <rsaref.h>

#include "psr.h"


int
main(int argc, char *argv[])
{
  dauthr_key public_key;
  dauthr_epass epass;
  R_RANDOM_STRUCT random_structure;
  char clearpass[MAXPASSLENGTH + 1];
  char *public_key_file = NULL;
  char epass_file[PATH_MAX];
  char logname[L_cuserid];
  int i;
  int errflg = 0;
  int verbose = 0;

  epass_file[0] = '\0';
  logname[0] = '\0';
  clearpass[0] = '\0';

  /* First we should process any commmand line arguements */
  while ((i = getopt(argc, argv, "e:v")) != -1)
    switch (i) {
    case 'e':
      (void) strcpy(epass_file, optarg);
      break;
    case 'v':
      verbose++;
      break;
    case '?':
      errflg++;
    }
  if (optind == (argc - 1))
    public_key_file = strdup(argv[optind++]);
  else
    errflg++;
  if (errflg) {
    (void) fprintf(stderr,
		   "usage: pwstore [-v]"
		   " [-e encryptedpasswdfile ] publickeyfile\n");
    exit(2);
  }
  /* find out who we are */

  if (cuserid(logname) == NULL) {
    (void) fprintf(stderr, "Couldn't find your login name, "
		   "exiting\n");
    exit(1);
  }
  /* Load the public keyfile */
  if (LoadKeyFile(&public_key, public_key_file)) {
    (void) fprintf(stderr, "Couldn't load the public key,"
		   " exiting\n");
    exit(1);
  }
  if (verbose)
    (void) printf("Public key file loaded from %s\n",
		  public_key_file);


  /* Find the Encrypted password file (if not assigned) */

  if (strlen(epass_file) == 0)
    if (DefaultEPassFile(epass_file, logname, public_key.keyname)) {
      (void) fprintf(stderr, "Couldn't locate your encrypted"
		     " password file, exiting\n");
      exit(1);
    }
  if (verbose)
    (void) printf("Encrypted password file is %s\n",
		  epass_file);

  (void) strncpy(clearpass, getpassphrase("Enter your AFS password: "),
		 MAXPASSLENGTH);

  /* Initialize the Random Structure */

  if (verbose)
    (void) printf("Initializing random data...\n");
  if (InitRandomStructure(&random_structure)) {
    (void) fprintf(stderr, "Couldn't initialize random"
		   " structure\n");
    exit(1);
  }
  if (verbose)
    (void) printf("Random structure initialized\n");


  /* Now call the crypto routine to encrypt the password. */
  if (EncryptPassword(&epass, clearpass, &public_key, &random_structure)) {
    (void) fprintf(stderr, "Password encryption for %s failed, "
		   "exiting\n", public_key.keyname);
    exit(1);
  }
  if (verbose)
    (void) printf("Encryption for %s successful\n",
		  public_key.keyname);


  /* And Store the encrypted password */
  if (SaveEPassFile(&epass, epass_file)) {
    (void) fprintf(stderr, "Failed to store ecrypted password"
		   " for %s\n", public_key.keyname);
    exit(1);
  }
  if (verbose)
    (void) printf("Encrypted password stored in %s\n",
		  epass_file);


  /* Destroy the random structure */

  DestroyRandomStructure(&random_structure);

  return 0;
}
